"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from honeyhive import utils
from typing import List, Optional


@dataclasses.dataclass
class DatasetUpdateMetadata:
    r"""Updated metadata to track for the dataset"""
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class DatasetUpdate:
    dataset_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('dataset_id') }})
    r"""The unique identifier of the dataset being updated"""
    datapoints: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('datapoints'), 'exclude': lambda f: f is None }})
    r"""Updated list of datapoint ids for the dataset"""
    description: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('description'), 'exclude': lambda f: f is None }})
    r"""Updated description for the dataset"""
    linked_evals: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('linked_evals'), 'exclude': lambda f: f is None }})
    r"""Updated list of unique evaluation run ids to be associated with this dataset"""
    metadata: Optional[DatasetUpdateMetadata] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    r"""Updated metadata to track for the dataset"""
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    r"""Updated name for the dataset"""
    num_points: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('num_points'), 'exclude': lambda f: f is None }})
    saved: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('saved'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http


@dataclasses.dataclass
class CreateToolSecurity:
    bearer_auth: str = dataclasses.field(metadata={'security': { 'scheme': True, 'type': 'http', 'sub_type': 'bearer', 'field_name': 'Authorization' }})
    



@dataclasses.dataclass
class CreateToolResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ...models.components import configuration as components_configuration
from enum import Enum
from typing import List, Optional

class Type(str, Enum):
    r"""Configuration type - \\"LLM\\" or \\"pipeline\\" """
    LLM = 'LLM'
    PIPELINE = 'pipeline'


@dataclasses.dataclass
class GetConfigurationsRequest:
    project_name: str = dataclasses.field(metadata={'query_param': { 'field_name': 'project_name', 'style': 'form', 'explode': True }})
    r"""Project name for configuration"""
    type: Type = dataclasses.field(metadata={'query_param': { 'field_name': 'type', 'style': 'form', 'explode': True }})
    r"""Configuration type - \\"LLM\\" or \\"pipeline\\" """
    



@dataclasses.dataclass
class GetConfigurationsResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    configurations: Optional[List[components_configuration.Configuration]] = dataclasses.field(default=None)
    r"""An array of configurations"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from honeyhive import utils
from typing import List, Optional


@dataclasses.dataclass
class Config:
    r"""Associated configuration JSON for the event - model name, vector index name, etc"""
    


class EventType(str, Enum):
    r"""Specify whether the event is of \\"model\\", \\"tool\\" or \\"chain\\" type"""
    MODEL = 'model'
    TOOL = 'tool'
    CHAIN = 'chain'


@dataclasses.dataclass
class Feedback:
    r"""Any user feedback provided for the event output"""
    



@dataclasses.dataclass
class CreateEventRequestInputs:
    r"""Input JSON given to the event - prompt, chunks, etc"""
    



@dataclasses.dataclass
class CreateEventRequestMetadata:
    r"""Any system or application metadata associated with the event"""
    



@dataclasses.dataclass
class Metrics:
    r"""Any values computed over the output of the event"""
    



@dataclasses.dataclass
class Outputs:
    r"""Final output JSON of the event"""
    



@dataclasses.dataclass
class CreateEventRequestUserProperties:
    r"""Any user properties associated with the event"""
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CreateEventRequest:
    event_name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_name') }})
    r"""Name of the event"""
    event_type: EventType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event_type') }})
    r"""Specify whether the event is of \\"model\\", \\"tool\\" or \\"chain\\" type"""
    project: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('project') }})
    r"""Project associated with the event"""
    source: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('source') }})
    r"""Source of the event - production, staging, etc"""
    children_ids: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('children_ids'), 'exclude': lambda f: f is None }})
    r"""Id of events that are nested within the event"""
    config: Optional[Config] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('config'), 'exclude': lambda f: f is None }})
    r"""Associated configuration JSON for the event - model name, vector index name, etc"""
    duration: Optional[float] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('duration'), 'exclude': lambda f: f is None }})
    r"""How long the event took in milliseconds"""
    end_time: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('end_time'), 'exclude': lambda f: f is None }})
    r"""UTC timestamp (in milliseconds) for the event end"""
    error: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('error'), 'exclude': lambda f: f is None }})
    r"""Any error description if event failed"""
    feedback: Optional[Feedback] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('feedback'), 'exclude': lambda f: f is None }})
    r"""Any user feedback provided for the event output"""
    inputs: Optional[CreateEventRequestInputs] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('inputs'), 'exclude': lambda f: f is None }})
    r"""Input JSON given to the event - prompt, chunks, etc"""
    metadata: Optional[CreateEventRequestMetadata] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    r"""Any system or application metadata associated with the event"""
    metrics: Optional[Metrics] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metrics'), 'exclude': lambda f: f is None }})
    r"""Any values computed over the output of the event"""
    outputs: Optional[Outputs] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('outputs'), 'exclude': lambda f: f is None }})
    r"""Final output JSON of the event"""
    parent_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('parent_id'), 'exclude': lambda f: f is None }})
    r"""Id of the parent event if nested"""
    start_time: Optional[float] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('start_time'), 'exclude': lambda f: f is None }})
    r"""UTC timestamp (in milliseconds) for the event start"""
    user_properties: Optional[CreateEventRequestUserProperties] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('user_properties'), 'exclude': lambda f: f is None }})
    r"""Any user properties associated with the event"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .createconfiguration import *
from .createdatapoint import *
from .createproject import *
from .createtool import *
from .delete_datasets import *
from .delete_events_event_id_ import *
from .delete_metrics import *
from .deleteconfiguration import *
from .deletedatapoint import *
from .deleteproject import *
from .deletesession import *
from .deletetool import *
from .get_datasets import *
from .get_metrics import *
from .getconfigurations import *
from .getdatapoint import *
from .getdatapoints import *
from .getprojects import *
from .getsession import *
from .gettools import *
from .post_datasets import *
from .post_events import *
from .post_metrics import *
from .processeventtrace import *
from .put_datasets import *
from .put_events import *
from .put_metrics import *
from .startsession import *
from .updateconfiguration import *
from .updatedatapoint import *
from .updateproject import *
from .updatetool import *

__all__ = ["CreateConfigurationResponse","CreateDatapointResponse","CreateDatapointResponseBody","CreateProjectResponse","CreateToolResponse","CreateToolSecurity","DeleteConfigurationRequest","DeleteConfigurationResponse","DeleteDatapointRequest","DeleteDatapointResponse","DeleteDatasetsRequest","DeleteDatasetsResponse","DeleteEventsEventIDRequest","DeleteEventsEventIDResponse","DeleteMetricsRequest","DeleteMetricsResponse","DeleteProjectRequest","DeleteProjectResponse","DeleteSessionRequest","DeleteSessionResponse","DeleteToolRequest","DeleteToolResponse","Feedback","GetConfigurationsRequest","GetConfigurationsResponse","GetDatapointRequest","GetDatapointResponse","GetDatapointsRequest","GetDatapointsResponse","GetDatapointsResponseBody","GetDatasetsQueryParamType","GetDatasetsRequest","GetDatasetsResponse","GetDatasetsResponseBody","GetMetricsRequest","GetMetricsResponse","GetProjectsRequest","GetProjectsResponse","GetSessionRequest","GetSessionResponse","GetToolsResponse","GetToolsSecurity","Metadata","Metrics","Outputs","PostDatasetsResponse","PostDatasetsResponseBody","PostDatasetsResult","PostEventsRequestBody","PostEventsResponse","PostEventsResponseBody","PostMetricsResponse","ProcessEventTraceRequest","ProcessEventTraceRequestBody","ProcessEventTraceResponse","ProcessEventTraceResponseBody","PutDatasetsResponse","PutEventsRequestBody","PutEventsResponse","PutMetricsResponse","QueryParamType","Result","StartSessionRequestBody","StartSessionResponse","StartSessionResponseBody","Type","UpdateConfigurationRequest","UpdateConfigurationResponse","UpdateDatapointRequest","UpdateDatapointResponse","UpdateProjectResponse","UpdateToolResponse","UpdateToolSecurity"]



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from honeyhive import utils
from typing import Any, Dict, List, Optional


@dataclasses.dataclass
class UpdateDatapointRequestHistory:
    pass


@dataclasses.dataclass
class UpdateDatapointRequestInputs:
    r"""Arbitrary JSON object containing the inputs for the datapoint"""
    



@dataclasses.dataclass
class UpdateDatapointRequestMetadata:
    r"""Any additional metadata for the datapoint"""
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class UpdateDatapointRequest:
    ground_truth: Optional[Dict[str, Any]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('ground_truth'), 'exclude': lambda f: f is None }})
    r"""Expected output JSON object for the datapoint"""
    history: Optional[List[UpdateDatapointRequestHistory]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('history'), 'exclude': lambda f: f is None }})
    r"""Update history for the datapoint"""
    inputs: Optional[UpdateDatapointRequestInputs] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('inputs'), 'exclude': lambda f: f is None }})
    r"""Arbitrary JSON object containing the inputs for the datapoint"""
    linked_datasets: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('linked_datasets'), 'exclude': lambda f: f is None }})
    r"""Ids of all datasets that include the datapoint"""
    linked_evals: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('linked_evals'), 'exclude': lambda f: f is None }})
    r"""Ids of evaluations where the datapoint is included"""
    metadata: Optional[UpdateDatapointRequestMetadata] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    r"""Any additional metadata for the datapoint"""
    saved: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('saved'), 'exclude': lambda f: f is None }})
    r"""Whether the datapoint is saved or detected"""
    type: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type'), 'exclude': lambda f: f is None }})
    r"""evaluation or event - specify the type of usage"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from dataclasses_json import Undefined, dataclass_json
from honeyhive import utils
from typing import Any, Dict, List, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class ProcessEventTraceRequestBody:
    logs: Optional[List[Dict[str, Any]]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('logs'), 'exclude': lambda f: f is None }})
    



@dataclasses.dataclass
class ProcessEventTraceRequest:
    session_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'session_id', 'style': 'simple', 'explode': False }})
    r"""The ID of the session to which this event trace belongs"""
    request_body: ProcessEventTraceRequestBody = dataclasses.field(metadata={'request': { 'media_type': 'application/json' }})
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class ProcessEventTraceResponseBody:
    r"""Successfully processed the event traces"""
    result: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('result'), 'exclude': lambda f: f is None }})
    



@dataclasses.dataclass
class ProcessEventTraceResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: Optional[ProcessEventTraceResponseBody] = dataclasses.field(default=None)
    r"""Successfully processed the event traces"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from honeyhive import utils
from typing import Any, Dict, List, Optional


@dataclasses.dataclass
class CreateDatapointRequestInputs:
    r"""Arbitrary JSON object containing the inputs for the datapoint"""
    



@dataclasses.dataclass
class CreateDatapointRequestMetadata:
    r"""Any additional metadata for the datapoint"""
    


class CreateDatapointRequestType(str, Enum):
    r"""Type of datapoint - \\"evaluation\\" or \\"fine-tuning\\" """
    EVALUATION = 'evaluation'
    FINE_TUNING = 'fine-tuning'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CreateDatapointRequest:
    inputs: CreateDatapointRequestInputs = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('inputs') }})
    r"""Arbitrary JSON object containing the inputs for the datapoint"""
    project: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('project') }})
    r"""UUID for the project to which the datapoint belongs"""
    type: CreateDatapointRequestType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type') }})
    r"""Type of datapoint - \\"evaluation\\" or \\"fine-tuning\\" """
    ground_truth: Optional[Dict[str, Any]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('ground_truth'), 'exclude': lambda f: f is None }})
    r"""Expected output JSON object for the datapoint"""
    linked_datasets: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('linked_datasets'), 'exclude': lambda f: f is None }})
    r"""Ids of all datasets that include the datapoint"""
    linked_evals: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('linked_evals'), 'exclude': lambda f: f is None }})
    r"""Ids of evaluations where the datapoint is included"""
    linked_event: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('linked_event'), 'exclude': lambda f: f is None }})
    r"""Event id for the event from which the datapoint was created"""
    metadata: Optional[CreateDatapointRequestMetadata] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    r"""Any additional metadata for the datapoint"""
    saved: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('saved'), 'exclude': lambda f: f is None }})
    r"""Whether the datapoint is saved or detected"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from honeyhive import utils
from typing import Optional

class ProjectType(str, Enum):
    COMPLETIONS = 'completions'
    CHAT = 'chat'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Project:
    description: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('description') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    type: ProjectType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type') }})
    id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .configuration import *
from .createdatapointrequest import *
from .createdatasetrequest import *
from .createeventrequest import *
from .createprojectrequest import *
from .datapoint import *
from .dataset import *
from .datasetupdate import *
from .event import *
from .metric import *
from .metricedit import *
from .project import *
from .security import *
from .sessionstartrequest import *
from .tool import *
from .toolupdate import *
from .updatedatapointrequest import *

__all__ = ["CallType","Config","Configuration","CreateDatapointRequest","CreateDatapointRequestInputs","CreateDatapointRequestMetadata","CreateDatapointRequestType","CreateDatasetRequest","CreateDatasetRequestMetadata","CreateDatasetRequestType","CreateEventRequest","CreateEventRequestInputs","CreateEventRequestMetadata","CreateEventRequestUserProperties","CreateProjectRequest","CreateProjectRequestType","Datapoint","Dataset","DatasetType","DatasetUpdate","DatasetUpdateMetadata","Event","EventConfig","EventEventType","EventFeedback","EventInputs","EventMetadata","EventMetrics","EventOutputs","EventType","EventUserProperties","Feedback","FunctionCallParams","History","Hyperparameters","Inputs","Metadata","Metric","MetricEdit","MetricEditEventType","MetricEditReturnType","MetricEditThreshold","MetricEditType","MetricType","Metrics","Outputs","Parameters","PipelineType","Project","ProjectType","ReturnType","Security","SelectedFunctions","SessionStartRequest","SessionStartRequestConfig","SessionStartRequestFeedback","SessionStartRequestInputs","SessionStartRequestMetadata","SessionStartRequestMetrics","SessionStartRequestOutputs","SessionStartRequestUserProperties","Threshold","Tool","ToolParameters","ToolType","ToolUpdate","ToolUpdateParameters","Type","UpdateDatapointRequest","UpdateDatapointRequestHistory","UpdateDatapointRequestInputs","UpdateDatapointRequestMetadata","UserProperties"]



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http


@dataclasses.dataclass
class PostMetricsResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    




# pylint: skip-file
import os
import uuid
from collections import defaultdict
from datetime import datetime
from enum import Enum
from typing import Any, Callable, Dict, List, Optional, Tuple

import requests
from llama_index.core.callbacks.base import BaseCallbackHandler
from llama_index.core.callbacks.schema import TIMESTAMP_FORMAT, CBEvent, CBEventType
from llama_index.core.callbacks.token_counting import get_llm_token_counts
from llama_index.core.utilities.token_counting import TokenCounter

from .langchain_tracer import Config, Log, log_to_dict


class HHEventType(str, Enum):
    MODEL = "model"
    CHAIN = "chain"
    TOOL = "tool"
    GENERIC = "generic"


class HoneyHiveLlamaIndexTracer(BaseCallbackHandler):
    _base_url: str = "https://api.honeyhive.ai"
    _headers: Dict[str, Any] = {"Content-Type": "application/json"}
    # Retrieve the API key from the environment variable
    _env_api_key = os.getenv("HONEYHIVE_API_KEY")

    def __init__(
        self,
        project: str,
        name: Optional[str] = None,
        source: Optional[str] = None,
        user_properties: Optional[Dict[str, Any]] = None,
        tokenizer: Optional[TokenCounter] = None,
        event_starts_to_ignore: Optional[List[CBEventType]] = None,
        event_ends_to_ignore: Optional[List[CBEventType]] = None,
        api_key: Optional[str] = None,
    ) -> None:
        if self._env_api_key:
            api_key = self._env_api_key
        elif not api_key:
            raise ValueError(
                "HoneyHive API key is not set! Please set the HONEYHIVE_API_KEY environment variable or pass in the api_key value."
            )

        if api_key:
            self._headers["Authorization"] = f"Bearer {api_key}"

        self.event_starts_to_ignore = event_starts_to_ignore or []
        self.event_ends_to_ignore = event_ends_to_ignore or []
        self._event_pairs_by_id: Dict[str, List[CBEvent]] = defaultdict(list)
        self._cur_trace_id: Optional[str] = None
        self._trace_map: Dict[str, List[str]] = defaultdict(list)
        self.tokenizer = (
            TokenCounter(tokenizer=tokenizer) if tokenizer else TokenCounter()
        )

        self.name = name
        self.project = project
        self.source = source
        self.user_properties = user_properties

    def _start_new_session(self):
        self.session_id = str(uuid.uuid4())
        body = {
            "project": self.project,
            "source": self.source,
            "session_name": self.name,
            "session_id": self.session_id,
            "user_properties": self.user_properties,
        }

        session_response = requests.post(
            url=f"{self._base_url}/session/start",
            headers=self._headers,
            json=body,
        )

        res = session_response.json()
        self.session_id = res["session_id"]

    def on_event_start(
        self,
        event_type: CBEventType,
        payload: Optional[Dict[str, Any]] = None,
        event_id: str = "",
        **kwargs: Any,
    ) -> str:
        """Store event start data by event type.

        Args:
            event_type (CBEventType): event type to store.
            payload (Optional[Dict[str, Any]]): payload to store.
            event_id (str): event id to store.

        """
        event = CBEvent(event_type, payload=payload, id_=event_id)
        self._event_pairs_by_id[event.id_].append(event)
        return event.id_

    def on_event_end(
        self,
        event_type: CBEventType,
        payload: Optional[Dict[str, Any]] = None,
        event_id: str = "",
        **kwargs: Any,
    ) -> None:
        """Store event end data by event type.

        Args:
            event_type (CBEventType): event type to store.
            payload (Optional[Dict[str, Any]]): payload to store.
            event_id (str): event id to store.

        """
        event = CBEvent(event_type, payload=payload, id_=event_id)
        self._event_pairs_by_id[event.id_].append(event)
        self._trace_map = defaultdict(list)

    def start_trace(self, trace_id: Optional[str] = None) -> None:
        """Launch a trace."""
        self._trace_map = defaultdict(list)
        self._cur_trace_id = trace_id
        self._start_time = datetime.now()

    def end_trace(
        self,
        trace_id: Optional[str] = None,
        trace_map: Optional[Dict[str, List[str]]] = None,
    ) -> None:
        self._trace_map = trace_map or defaultdict(list)
        self._end_time = datetime.now()

        self.log_trace()

    def log_trace(self) -> None:
        try:
            child_nodes = self._trace_map.get("root")
            if child_nodes:
                root_span = self._convert_event_pair_to_log(
                    self._event_pairs_by_id.get(child_nodes[0]),
                    trace_id=self._cur_trace_id if len(child_nodes) > 1 else None,
                )
                if len(child_nodes) == 1:
                    child_nodes = self._trace_map.get(child_nodes[0])
                    root_span = self._build_trace(child_nodes, root_span)
                else:
                    root_span = self._build_trace(child_nodes, root_span)
                if root_span and root_span.event_type == "chain":
                    self._post_trace(root_span)
        except Exception:
            # Silently ignore errors to not break user code
            pass

    def _build_trace(
        self, events: List[str], span: Log, parent_id: Optional[str] = None
    ) -> Log:
        """Build the trace tree from the trace map."""
        for child_event in events:
            child_span = self._convert_event_pair_to_log(
                self._event_pairs_by_id[child_event], parent_id=parent_id
            )
            child_span = self._build_trace(
                self._trace_map[child_event], child_span, child_span.event_id
            )
            if span.children is None:
                span.children = [child_span]
            else:
                span.children.append(child_span)
        return span

    def _convert_event_pair_to_log(
        self,
        event_pair: List[CBEvent],
        parent_id: Optional[str] = None,
        trace_id: Optional[str] = None,
    ) -> Log:
        """Convert a pair of events to a HoneyHive log."""
        start_time_us, end_time_us, end_time = self._get_time_in_us(event_pair)

        if trace_id is None:
            event_type = event_pair[0].event_type
            span_kind = self._map_event_type(event_type)
        else:
            event_type = trace_id  # type: ignore
            span_kind = "generic"

        root_log = Log(
            project=self.project,
            source=self.source,
            event_id=str(uuid.uuid4()),
            inputs={},
            outputs={},
            children=None,
            error=None,
            parent_id=parent_id,
            config=Config(),
            event_name=f"{getattr(event_type, 'value', event_type)}",
            event_type=span_kind,
            start_time=start_time_us,
            end_time=end_time_us,
            duration=(end_time_us - start_time_us) / 1000,
        )

        inputs, outputs, root_log = self._add_payload_to_log(root_log, event_pair)
        root_log.inputs = inputs
        root_log.outputs = outputs

        return root_log

    def _map_event_type(self, event_type: CBEventType) -> str:
        """Map a CBEventType to a HoneyHive event type."""
        if event_type in [
            CBEventType.EMBEDDING,
            CBEventType.LLM,
            CBEventType.SYNTHESIZE,
        ]:
            hh_event_type = HHEventType.MODEL
        elif event_type in [
            CBEventType.QUERY,
            CBEventType.TREE,
            CBEventType.SUB_QUESTION,
        ]:
            hh_event_type = HHEventType.CHAIN
        elif event_type == CBEventType.RETRIEVE:
            hh_event_type = HHEventType.TOOL
        else:
            hh_event_type = HHEventType.GENERIC

        return hh_event_type

    def _add_payload_to_log(
        self, span: Log, event_pair: List[CBEvent]
    ) -> Tuple[Optional[Dict[str, Any]], Optional[Dict[str, Any]], Log]:
        """Add the event's payload to the span."""
        assert len(event_pair) == 2
        event_type = event_pair[0].event_type
        inputs = None
        outputs = {}

        if event_type == CBEventType.NODE_PARSING:
            # TODO: disabled full detailed inputs/outputs due to UI lag
            inputs, outputs = self._handle_node_parsing_payload(event_pair)
        elif event_type == CBEventType.LLM:
            inputs, outputs, span = self._handle_llm_payload(event_pair, span)
        elif event_type == CBEventType.QUERY:
            inputs, outputs = self._handle_query_payload(event_pair)
        elif event_type == CBEventType.EMBEDDING:
            inputs, outputs = self._handle_embedding_payload(event_pair)
        elif event_type == CBEventType.RETRIEVE:
            inputs, outputs = self._handle_retrieve_payload(event_pair)

        return inputs, outputs, span

    def _handle_retrieve_payload(
        self, event_pair: List[CBEvent]
    ) -> Tuple[Dict[str, Any], Dict[str, Any]]:
        inputs = {"query_str": event_pair[0].payload["query_str"]}
        chunks = []
        for node in event_pair[1].payload["nodes"]:
            chunks.append({"score": node.score, "text": node.node.text})
        outputs = {"chunks": chunks}
        return inputs, outputs

    def _handle_node_parsing_payload(
        self, event_pair: List[CBEvent]
    ) -> Tuple[Dict[str, Any], Dict[str, Any]]:
        """Handle the payload of a NODE_PARSING event."""
        inputs = event_pair[0].payload
        outputs = event_pair[-1].payload

        if inputs and "documents" in inputs:
            documents = inputs.pop("documents")
            inputs["num_documents"] = len(documents)

        if outputs and "nodes" in outputs:
            nodes = outputs.pop("nodes")
            outputs["num_nodes"] = len(nodes)

        return inputs or {}, outputs or {}

    def _handle_llm_payload(
        self, event_pair: List[CBEvent], span: Log
    ) -> Tuple[Dict[str, Any], Dict[str, Any], Log]:
        """Handle the payload of a LLM event."""
        inputs = event_pair[0].payload
        outputs = event_pair[-1].payload

        assert isinstance(inputs, dict) and isinstance(outputs, dict)

        # Get `original_template` from Prompt
        if "formatted_prompt" in inputs:
            inputs["formatted_prompt"] = inputs["formatted_prompt"]

        # Format messages
        if "messages" in inputs:
            inputs["messages"] = "\n".join([str(x) for x in inputs["messages"]])

        token_counts = get_llm_token_counts(self.tokenizer, outputs)
        metadata = {
            "formatted_prompt_tokens_count": token_counts.prompt_token_count,
            "prediction_tokens_count": token_counts.completion_token_count,
            "total_tokens_used": token_counts.total_token_count,
        }
        span.metadata = metadata

        # Make `response` part of `outputs`
        if "response" in outputs:
            outputs = {"response": outputs["response"]}
        elif "completion" in outputs:
            outputs = {"response": outputs["completion"].text}

        return inputs, outputs, span

    def _handle_query_payload(
        self, event_pair: List[CBEvent]
    ) -> Tuple[Optional[Dict[str, Any]], Dict[str, Any]]:
        """Handle the payload of a QUERY event."""
        inputs = event_pair[0].payload
        outputs = event_pair[-1].payload

        if outputs:
            response = outputs["response"]

            if type(response).__name__ == "Response":
                response = response.response
            elif type(response).__name__ == "StreamingResponse":
                response = response.get_response().response
        else:
            response = " "

        outputs = {"response": response}

        return inputs, outputs

    def _handle_embedding_payload(
        self,
        event_pair: List[CBEvent],
    ) -> Tuple[Optional[Dict[str, Any]], Dict[str, Any]]:
        event_pair[0].payload
        outputs = event_pair[-1].payload

        chunks = []
        if outputs:
            chunks = outputs.get("chunks", [])

        return {}, {"chunks": chunks}

    def _get_time_in_us(self, event_pair: List[CBEvent]) -> Tuple[int, int]:
        """Get the start and end time of an event pair in microseconds."""
        start_time = datetime.strptime(event_pair[0].time, TIMESTAMP_FORMAT)
        end_time = datetime.strptime(event_pair[1].time, TIMESTAMP_FORMAT)

        start_time_in_ms = int(
            (start_time - datetime(1970, 1, 1)).total_seconds() * 1000000
        )
        end_time_in_ms = int(
            (end_time - datetime(1970, 1, 1)).total_seconds() * 1000000
        )

        return start_time_in_ms, end_time_in_ms, end_time

    def _post_trace(self, root_log: Log) -> None:
        self._start_new_session()
        root_log = log_to_dict(root_log)
        root_log["session_id"] = str(uuid.uuid4())
        trace_response = requests.post(
            url=f"{self._base_url}/session/{self.session_id}/traces",
            json={"logs": [root_log]},
            headers=self._headers,
        )
        if trace_response.status_code != 200:
            raise Exception(
                f"Failed to post trace to HoneyHive with status code {trace_response.status_code}"
            )

    def finish(self) -> None:
        pass



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from honeyhive import utils
from typing import Any, Dict, List, Optional

class Type(str, Enum):
    r"""Type of the configuration - \\"LLM\\" or \\"pipeline\\" - \\"LLM\\" by default"""
    LLM = 'LLM'
    PIPELINE = 'pipeline'

class CallType(str, Enum):
    r"""Type of API calling - \\"chat\\" or \\"completion\\" """
    CHAT = 'chat'
    COMPLETION = 'completion'


@dataclasses.dataclass
class Hyperparameters:
    r"""Model-specific hyperparameters"""
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class SelectedFunctions:
    id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id'), 'exclude': lambda f: f is None }})
    r"""UUID of the function"""
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    r"""Name of the function"""
    description: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('description'), 'exclude': lambda f: f is None }})
    r"""Description of the function"""
    parameters: Optional[Dict[str, Any]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('parameters'), 'exclude': lambda f: f is None }})
    r"""Parameters for the function"""
    


class FunctionCallParams(str, Enum):
    r"""Function calling mode - \\"none\\", \\"auto\\" or \\"force\\" """
    NONE = 'none'
    AUTO = 'auto'
    FORCE = 'force'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Parameters:
    call_type: CallType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('call_type') }})
    r"""Type of API calling - \\"chat\\" or \\"completion\\" """
    model: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('model') }})
    r"""Model unique name"""
    hyperparameters: Optional[Hyperparameters] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('hyperparameters'), 'exclude': lambda f: f is None }})
    r"""Model-specific hyperparameters"""
    selected_functions: Optional[List[SelectedFunctions]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('selectedFunctions'), 'exclude': lambda f: f is None }})
    r"""List of functions to be called by the model, refer to OpenAI schema for more details"""
    function_call_params: Optional[FunctionCallParams] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('functionCallParams'), 'exclude': lambda f: f is None }})
    r"""Function calling mode - \\"none\\", \\"auto\\" or \\"force\\" """
    force_function: Optional[Dict[str, Any]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('forceFunction'), 'exclude': lambda f: f is None }})
    r"""Force function-specific parameters"""
    



@dataclasses.dataclass
class UserProperties:
    r"""Details of user who created the configuration"""
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Configuration:
    project: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('project') }})
    r"""ID of the project to which this configuration belongs"""
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    r"""Name of the configuration"""
    provider: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('provider') }})
    r"""Name of the provider - \\"openai\\", \\"anthropic\\", etc."""
    type: Optional[Type] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type'), 'exclude': lambda f: f is None }})
    r"""Type of the configuration - \\"LLM\\" or \\"pipeline\\" - \\"LLM\\" by default"""
    parameters: Optional[Parameters] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('parameters'), 'exclude': lambda f: f is None }})
    user_properties: Optional[UserProperties] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('user_properties'), 'exclude': lambda f: f is None }})
    r"""Details of user who created the configuration"""
    

